package in.mobiocean.mo_a5_plugin;

import android.app.Activity;

import com.telpo.tps550.api.TelpoException;
import com.telpo.tps550.api.nfc.Nfc;
import com.telpo.tps550.api.util.StringUtil;

public class NfcSample {

    private final Nfc nfc;

    private Thread readThread;
    private ReadCallback callback;

    public NfcSample(Activity activity, ReadCallback callback) {
        nfc = new Nfc(activity);
        this.callback = callback;
    }

    public void updateCallBack(ReadCallback callback){
        this.callback = callback;
    }

    public int startNfc() {
        try {
            nfc.open();
            return 0;
        } catch (TelpoException e) {
            e.printStackTrace();
        }
        return -1;
    }

    public int closeNfc() {
        try {
            nfc.close();
            return 0;
        } catch (TelpoException e) {
            e.printStackTrace();
        }
        return -1;
    }

    public void readCard() {
        try {
            readThread = new ReadThread();
            readThread.start();
        } catch (Exception e) {
            e.printStackTrace();
            if(callback != null)
                callback.readFailure(e);
        }
    }

    public int m1CardAuthenticate(byte blockNo, int[] passwd) {
        try {
            byte[] newArray = new byte[passwd.length];
            for(int i=0; i<passwd.length; i++){
                newArray[i] = (byte) passwd[i];
            }
            nfc.m1_authenticate(blockNo, (byte)0x0A, newArray);
            return 0;
        } catch (TelpoException e) {
            e.printStackTrace();
        }
        return -1;
    }

    public int writeValueData(byte blockNo, String valueStr) {
        byte[] valueData = null;
        valueData = toByteArray(valueStr);
        try {
            nfc.m1_write_block(blockNo, valueData, valueData.length);
            return 0;
        } catch (TelpoException e) {
            e.printStackTrace();
        }
        return -1;
    }

    public String readValueData(byte blockNo) {
        byte[] data = null;
        try {
            data = nfc.m1_read_value(blockNo);
        } catch (TelpoException e) {
            e.printStackTrace();
        }
        if (null == data) {
            return null;
        } else {
            return StringUtil.toHexString(data);
        }
    }

    public int m1HaltOperation() {
        try {
            nfc.halt();
            return 0;
        } catch (TelpoException e) {
            e.printStackTrace();
        }
        return -1;
    }

    private class ReadThread extends Thread {
        byte[] nfcData = null;
        @Override
        public void run() {
            try {
                nfcData = nfc.activate(10 * 1000);
                if (null != nfcData) {
                    byte[] uid_data = nfcData;
                    if (uid_data[0] == 0x41) {
                        byte[] atqa = new byte[2];
                        byte[] sak = new byte[1];
                        byte[] uid = new byte[uid_data[5]];

                        System.arraycopy(uid_data, 2, atqa, 0, 2);
                        System.arraycopy(uid_data, 4, sak, 0, 1);
                        System.arraycopy(uid_data, 6, uid, 0, uid_data[5]);

                        if (uid_data[1] == 2) {
                            if(callback != null) callback.cardRead(
                                    StringUtil.toHexString(atqa),
                                    StringUtil.toHexString(sak),
                                    StringUtil.toHexString(uid));
                        } else {
                            if(callback != null) callback.unknownCard();
                        }
                    } else {
                        if(callback != null) callback.unknownCard();
                    }
                } else {
                    if(callback != null) callback.timeout();
                }
            } catch (TelpoException e) {
                if(callback != null) callback.readFailure(e);
            }
        }
    }

    private byte[] toByteArray(String hexString) {
        int hexStringLength = hexString.length();
        byte[] byteArray = null;
        int count = 0;
        char c;
        int i;

        // Count number of hex characters
        for (i = 0; i < hexStringLength; i++) {
            c = hexString.charAt(i);
            if (c >= '0' && c <= '9' || c >= 'A' && c <= 'F' || c >= 'a' && c <= 'f') {
                count++;
            }
        }

        byteArray = new byte[(count + 1) / 2];
        boolean first = true;
        int len = 0;
        int value;
        for (i = 0; i < hexStringLength; i++) {
            c = hexString.charAt(i);
            if (c >= '0' && c <= '9') {
                value = c - '0';
            } else if (c >= 'A' && c <= 'F') {
                value = c - 'A' + 10;
            } else if (c >= 'a' && c <= 'f') {
                value = c - 'a' + 10;
            } else {
                value = -1;
            }

            if (value >= 0) {

                if (first) {
                    byteArray[len] = (byte) (value << 4);
                } else {
                    byteArray[len] |= value;
                    len++;
                }
                first = !first;
            }
        }
        return byteArray;
    }
}
